@@ -1,22 +1,22 @@
 from __future__ import annotations
 
-import os
 from pathlib import Path
 from typing import List, NamedTuple, Optional, Tuple, Union
 
 import mecab_ko_dic
 
 import _mecab
+from mecab.utils import create_lattice, ensure_list, to_csv
+
+PathLike = Union[str, Path]
 
 mecabrc_path = Path(__file__).absolute().parent / "mecabrc"
+_rcfile_option = ["--rcfile", str(mecabrc_path)]
 
 
-def _create_lattice(sentence: str) -> _mecab.Lattice:
-    lattice = _mecab.Lattice()
-    lattice.add_request_type(_mecab.MECAB_ALLOCATE_SENTENCE)  # Required
-    lattice.set_sentence(sentence)
-
-    return lattice
+class Span(NamedTuple):
+    start: int
+    end: int
 
 
 class Feature(NamedTuple):
@@ -28,9 +28,10 @@ class Feature(NamedTuple):
     start_pos: Optional[str] = None
     end_pos: Optional[str] = None
     exprssion: Optional[str] = None
+    span: Optional[Span] = None
 
     @classmethod
-    def _from_node(cls, node: _mecab.Node) -> Feature:
+    def _from_node(cls, span: Tuple[int, int], node: _mecab.Node) -> Feature:
         # Reference:
         # - http://taku910.github.io/mecab/learn.html
         # - https://docs.google.com/spreadsheets/d/1-9blXKjtjeKZqsf4NzHeYJCrr49-nXeRF6D80udfcwY
@@ -40,23 +41,19 @@ class Feature(NamedTuple):
         values = node.feature.split(",")
         assert len(values) == 8
 
-        feature = {
-            field: value if value != "*" else None
-            for field, value in zip(Feature._fields, values)
-        }
+        feature = {field: value if value != "*" else None for field, value in zip(Feature._fields, values)}
 
         if feature["has_jongseong"] == "T":
             feature["has_jongseong"] = True
         elif feature["has_jongseong"] == "F":
             feature["has_jongseong"] = False
 
-        return cls(**feature)
+        return cls(**feature, span=Span(*span))
 
     def __str__(self) -> str:
-        feature = {
-            key: value if value is not None else "*"
-            for key, value in self._asdict().items()
-        }
+        feature = {key: value if value is not None else "*" for key, value in self._asdict().items()}
+        del feature["span"]  # mecab feature does not include span
+
         # True -> T / False -> F / * -> *
         feature["has_jongseong"] = str(feature["has_jongseong"])[0]
         return ",".join(feature.values())
@@ -69,37 +66,30 @@ class MeCabError(Exception):
 class MeCab:  # APIs are inspired by KoNLPy
     def __init__(
         self,
-        dictionary_path: Optional[os.PathLike] = None,
-        user_dictionay_path: Optional[Union[os.PathLike, List[os.PathLike]]] = None,
+        dictionary_path: Optional[PathLike] = None,
+        user_dictionay_path: Optional[Union[PathLike, List[PathLike]]] = None,
     ):
         if dictionary_path is None:
-            dictionary_path = str(mecab_ko_dic.dictionary_path)
+            dictionary_path = mecab_ko_dic.dictionary_path
 
-        user_dictionay_argument = []
-        if user_dictionay_path is not None:
-            if not isinstance(user_dictionay_path, list):
-                user_dictionay_path = [user_dictionay_path]
+        user_dictionay_path = ensure_list(user_dictionay_path)
 
-            user_dictionay_argument = [
-                "--userdic",
-                ",".join(f'"{path}"' for path in user_dictionay_path),
-            ]
+        dictionary_option = ["--dicdir", str(dictionary_path)]
+        user_dictionay_option = ["--userdic", to_csv(user_dictionay_path)] if user_dictionay_path else []
 
-        arguments = [
-            "--rcfile",
-            str(mecabrc_path),
-            "--dicdir",
-            dictionary_path,
-            *user_dictionay_argument,
+        options = [
+            *_rcfile_option,
+            *dictionary_option,
+            *user_dictionay_option,
         ]
-        self._tagger = _mecab.Tagger(arguments)
+        self._tagger = _mecab.Tagger(options)
 
     def parse(self, sentence: str) -> List[Tuple[str, Feature]]:
-        lattice = _create_lattice(sentence)
+        lattice = create_lattice(sentence)
         if not self._tagger.parse(lattice):
             raise MeCabError(self._tagger.what())
 
-        return [(node.surface, Feature._from_node(node)) for node in lattice]
+        return [(node.surface, Feature._from_node(span, node)) for span, node in lattice]
 
     def pos(self, sentence: str) -> List[Tuple[str, str]]:
         return [(surface, feature.pos) for surface, feature in self.parse(sentence)]
@@ -108,8 +98,4 @@ class MeCab:  # APIs are inspired by KoNLPy
         return [surface for surface, _ in self.parse(sentence)]
 
     def nouns(self, sentence: str) -> List[str]:
-        return [
-            surface
-            for surface, feature in self.parse(sentence)
-            if feature.pos.startswith("N")
-        ]
+        return [surface for surface, feature in self.parse(sentence) if feature.pos.startswith("N")]
